//    This file is part of JustMotion.

//    JustMotion is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.

//    JustMotion is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.

#include "event_loop.h"

EventLoop::EventLoop(shared_t *sharedRes, QThread *thr, QObject *parent) : QObject(parent)
{
    shared    = sharedRes;
    heartBeat = 2;
    loopTimer = 0;

    connect(thr, &QThread::started,  this, &EventLoop::init);
    connect(thr, &QThread::finished, this, &EventLoop::deleteLater);

    moveToThread(thr);
}

void EventLoop::init()
{
    loopTimer = new QTimer(this);

    connect(loopTimer, &QTimer::timeout, this, &EventLoop::loopSlot);

    loopTimer->setSingleShot(false);
    loopTimer->start(heartBeat * 1000);

    loopSlot();
}

void EventLoop::loopSlot()
{
    if (!exec())
    {
        loopTimer->stop(); QCoreApplication::exit(shared->retCode);
    }
}

bool EventLoop::wrOutVod(const evt_t &event)
{
    auto ret    = false;
    auto cnt    = 0;
    auto concat = shared->buffPath + "/live/" + event.timeStamp + ".ctmp";

    QFile file(concat, this);

    file.open(QFile::WriteOnly);

    for (auto &&vid : event.vidList)
    {
        qInfo() << "event_src: " << vid;

        if (QFile::exists(vid))
        {
            file.write(QString("file '" + vid + "'\n").toUtf8()); cnt++;
        }
        else
        {
            qInfo() << "warning: the event video clip does not exists";
        }
    }

    file.close();

    if (cnt == 0)
    {
        qCritical() << "err: none of the event hls clips exists, cancelling write out";

        QFile::remove(concat);
    }
    else
    {
        QStringList args;

        args << "-f";
        args << "concat";
        args << "-safe" << "0";
        args << "-i" << concat;
        args << "-c" << "copy";
        args << shared->recPath + "/" + event.timeStamp + shared->recExt;

        if (QProcess::execute("ffmpeg", args) == 0)
        {
            ret = true;
        }

        QFile::remove(concat);
    }

    return ret;
}

QString EventLoop::statusLine()
{
    if (loopTimer->isActive())
    {
        return "OK  ";
    }
    else
    {
        return "FAIL";
    }
}

bool EventLoop::exec()
{
    enforceMaxEvents(shared);
    enforceMaxImages(shared);
    enforceMaxClips(shared);

    if (!shared->recList.isEmpty())
    {
        auto event = shared->recList.takeFirst();

        qInfo() << "attempting write out of event: " << event.timeStamp;

        if (wrOutVod(event))
        {
            QStringList args;

            args << "convert";
            args << event.imgPath;
            args << shared->recPath + "/" + event.timeStamp + shared->thumbExt;

            QProcess::execute("magick", args);
        }
    }

    return shared->retCode == 0;
}
