//    This file is part of JustMotion.

//    JustMotion is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.

//    JustMotion is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.

#include "proc_control.h"

ProcControl::ProcControl(const QString &procFile, const QString &statDir, QCoreApplication *parent) : QObject(parent)
{
    fsMon       = new QFileSystemWatcher(this);
    file        = procFile;
    statPath    = statDir;
    closeOnZero = false;
    objCount    = 0;

    connect(fsMon, &QFileSystemWatcher::fileChanged, this, &ProcControl::procFileUpdated);
}

bool ProcControl::init()
{
    auto ret = false;

    if (!QFileInfo::exists(file))
    {
        QFile fObj(file);

        if (!fObj.open(QFile::WriteOnly))
        {
            QTextStream(stderr) << "err: Failed to open process control file for writing: " << file << " reason: " << fObj.errorString();
        }
        else
        {
            fObj.write("##");
            fObj.close();

            ret = true;
        }
    }
    else
    {
        QTextStream(stdout) << "wrn: " << file << " already exists so it will be removed, this will cause any other existing instance to close.";

        if (!QFile::remove(file))
        {
            QTextStream(stderr) << "err: Failed to remove process control file: " << file << " check permissions.";
        }
        else
        {
            thread()->sleep(3);

            ret = init();
        }
    }

    if (ret)
    {
        fsMon->addPath(file);
    }

    return ret;
}

void ProcControl::procFileUpdated(const QString &path)
{
    fsMon->removePath(path);

    QFile::remove(path);

    closeApp();
}

void ProcControl::closeApp()
{
    if (objCount == 0)
    {
        QDir(statPath).removeRecursively();

        QCoreApplication::instance()->quit();
    }
    else
    {
        closeOnZero = true;

        emit prepForClose();
    }
}

void ProcControl::objPlusOne()
{
    objCount++;
}

void ProcControl::objMinusOne()
{
    objCount--;

    if (closeOnZero && (objCount == 0))
    {
        closeApp();
    }
}
